! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
! Module: fits_utils
!
! Authors:  Masaaki Sakano - SSC/LUX
!
! Description: Utility library to handle FITS files.
!
! Comments: TODO:
!
! $Id: fits_utils.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module fits_utils

  use dal
  use types
  use errorHandling

  use dal_aux
  use array_utils

  implicit none 

  private 

  public getFitsVecStatInfoDouble		! Function
  public getFitsVecStatInfoSingle		! Function
  public getFitsVecStatInfoInt32		! Function
  public getFitsVecStatInfoInt16		! Function
  public getFitsVecStatInfoInt8			! Function

  public getFitsImgStatInfoDouble		! Function
  public getFitsImgStatInfoSingle		! Function
  public getFitsImgStatInfoInt32		! Function
  public getFitsImgStatInfoInt16		! Function
  public getFitsImgStatInfoInt8			! Function

  public getAnnularMask				! Function

  interface getFitsVecStatInfoDouble
    module procedure  getFitsVecStatDoubleName, getFitsVecStatDoubleNameSimp
    module procedure  getFitsVecStatDoubleSet,  getFitsVecStatDoubleSetSimp
    module procedure  getFitsVecStatDoubleTab,  getFitsVecStatDoubleTabSimp
    module procedure  getFitsVecStatDoubleCol,  getFitsVecStatDoubleColSimp
  end interface
  interface getFitsVecStatInfoSingle
    module procedure  getFitsVecStatSingleName, getFitsVecStatSingleNameSimp
    module procedure  getFitsVecStatSingleSet,  getFitsVecStatSingleSetSimp
    module procedure  getFitsVecStatSingleTab,  getFitsVecStatSingleTabSimp
    module procedure  getFitsVecStatSingleCol,  getFitsVecStatSingleColSimp
  end interface
  interface getFitsVecStatInfoInt32
    module procedure  getFitsVecStatInt32Name, getFitsVecStatInt32NameSimp
    module procedure  getFitsVecStatInt32Set,  getFitsVecStatInt32SetSimp
    module procedure  getFitsVecStatInt32Tab,  getFitsVecStatInt32TabSimp
    module procedure  getFitsVecStatInt32Col,  getFitsVecStatInt32ColSimp
  end interface
  interface getFitsVecStatInfoInt16
    module procedure  getFitsVecStatInt16Name, getFitsVecStatInt16NameSimp
    module procedure  getFitsVecStatInt16Set,  getFitsVecStatInt16SetSimp
    module procedure  getFitsVecStatInt16Tab,  getFitsVecStatInt16TabSimp
    module procedure  getFitsVecStatInt16Col,  getFitsVecStatInt16ColSimp
  end interface
  interface getFitsVecStatInfoInt8
    module procedure  getFitsVecStatInt8Name,  getFitsVecStatInt8NameSimp
    module procedure  getFitsVecStatInt8Set,   getFitsVecStatInt8SetSimp
    module procedure  getFitsVecStatInt8Tab,   getFitsVecStatInt8TabSimp
    module procedure  getFitsVecStatInt8Col,   getFitsVecStatInt8ColSimp
  end interface

  interface getFitsImgStatInfoDouble
    module procedure getFitsImgStatDoubleName, getFitsImgStatDoubleSet
  end interface
  interface getFitsImgStatInfoSingle
    module procedure getFitsImgStatSingleName, getFitsImgStatSingleSet
  end interface
  interface getFitsImgStatInfoInt32
    module procedure getFitsImgStatInt32Name, getFitsImgStatInt32Set
  end interface
  interface getFitsImgStatInfoInt16
    module procedure getFitsImgStatInt16Name, getFitsImgStatInt16Set
  end interface
  interface getFitsImgStatInfoInt8
    module procedure getFitsImgStatInt8Name, getFitsImgStatInt8Set
  end interface

  interface getAnnularMask
    module procedure getAnnularMaskDoubleName, getAnnularMaskDoubleSet
    module procedure getAnnularMaskSingleName, getAnnularMaskSingleSet
    ! module procedure getAnnularMaskInt32Name, getAnnularMaskInt32Set
    ! module procedure getAnnularMaskInt16Name, getAnnularMaskInt16Set
    ! module procedure getAnnularMaskInt8Name, getAnnularMaskInt8Set
  end interface

  integer, parameter :: strTypeLength = 255

contains 


  !---------------------------------------------------------------------
  function getFitsVecStatDoubleName(setTabName, colName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo for a given FITS table.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: setTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    character(1024) :: setName, tabName
    logical :: noColonFound
    type(DataSetT) :: set
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(setTabName, setName, tabName, noColonFound, useBlock=.false.)
    if (noColonFound .and. (.not. present(extNum))) then
      ! The argument setTabName MUST contain the table name unless extNum is given.
      call error('noTabNameInGetFitsVecStat', 'In ssclib/fits_utils, for setTabName='//trim(setTabName))
    end if

    strFileInfoOut = "FileName="//trim(setName)
    set = dataSet(trim(setName), READ)

    if (trim(tabName) == '') then
      aryStatInfo = getFitsVecStatDoubleSet(set, colName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    else
      aryStatInfo = getFitsVecStatDoubleSet(set, colName, tabName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    end if

    call release(set)

  end function getFitsVecStatDoubleName

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleNameSimp(fitsSetTabName, colName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: fitsSetTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatDoubleName(fitsSetTabName, colName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo)

  end function getFitsVecStatDoubleNameSimp

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleSet(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.
  ! Note if tabName is an empty string, it is ignored.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(TableT) :: tab
    logical :: isTableDefined
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    isTableDefined = .false.
    if (present(tabName)) then
      if (trim(tabName) .ne. '') then
        isTableDefined = .true.
        tab = table(inSet, trim(tabName))
        strFileInfoOut = "Table="//trim(tabName)
      end if
    end if

    if (isTableDefined) then
      ! Do nothing
    else if (present(extNum)) then
      tab = table(inSet, extNum)
      write(strFileInfoOut,'("Table-Ext=",i03)') extNum
    else
      ! Neither tabName nor extNum is specified.
      call error('noTabNameExtNum', 'Contact the code developper (ssclib/fits_utils).')
    end if

    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    aryStatInfo = getFitsVecStatDoubleTab(tab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(tab)

  end function getFitsVecStatDoubleSet

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleSetSimp(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatDoubleSet(inSet, colName, tabName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatDoubleSetSimp

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleTab(inTab, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(ColumnT) :: col
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    strFileInfoOut = "Column="//trim(colName)
    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    col = column(inTab, trim(colName), READ)

    aryStatInfo = getFitsVecStatDoubleCol(col &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(col)

  end function getFitsVecStatDoubleTab

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleTabSimp(inTab, colName, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatDoubleTab(inTab, colName &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatDoubleTabSimp

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleCol(inCol, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    real(double), pointer :: aryData(:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = dataType(inCol)	! dal
    if (iType .ne. REAL64) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into double in fits_utils/getFitsVecStatDouble"
      if (present(strFileInfo)) then
        errStr = trim(errStr)//" for "//trim(strFileInfo)
      else
        errStr = trim(errStr)//"."
      end if
      call message(AppMsg, Noisy)
    end if

    aryData => real64Data(inCol)
    aryStatInfo = getAryStatInfo(aryData, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

  end function getFitsVecStatDoubleCol

  !---------------------------------------------------------------------
  function getFitsVecStatDoubleColSimp(inCol, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatDoubleCol(inCol &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatDoubleColSimp


  !---------------------------------------------------------------------
  function getFitsVecStatSingleName(setTabName, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper, extNum &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo for a given FITS table.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: setTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    integer, intent(in), optional :: extNum
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    character(1024) :: setName, tabName
    logical :: noColonFound
    type(DataSetT) :: set
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(setTabName, setName, tabName, noColonFound, useBlock=.false.)
    if (noColonFound .and. (.not. present(extNum))) then
      ! The argument setTabName MUST contain the table name unless extNum is given.
      call error('noTabNameInGetFitsVecStat', 'In ssclib/fits_utils, for setTabName='//trim(setTabName))
    end if

    strFileInfoOut = "FileName="//trim(setName)
    set = dataSet(trim(setName), READ)

    if (trim(tabName) == '') then
      aryStatInfo = getFitsVecStatSingleSet(set, colName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    else
      aryStatInfo = getFitsVecStatSingleSet(set, colName, tabName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    end if

    call release(set)

  end function getFitsVecStatSingleName

  !---------------------------------------------------------------------
  function getFitsVecStatSingleNameSimp(fitsSetTabName, colName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: fitsSetTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatSingleName(fitsSetTabName, colName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo)

  end function getFitsVecStatSingleNameSimp

  !---------------------------------------------------------------------
  function getFitsVecStatSingleSet(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.
  ! Note if tabName is an empty string, it is ignored.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(TableT) :: tab
    logical :: isTableDefined
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    isTableDefined = .false.
    if (present(tabName)) then
      if (trim(tabName) .ne. '') then
        isTableDefined = .true.
        tab = table(inSet, trim(tabName))
        strFileInfoOut = "Table="//trim(tabName)
      end if
    end if

    if (isTableDefined) then
      ! Do nothing
    else if (present(extNum)) then
      tab = table(inSet, extNum)
      write(strFileInfoOut,'("Table-Ext=",i03)') extNum
    else
      ! Neither tabName nor extNum is specified.
      call error('noTabNameExtNum', 'Contact the code developper (ssclib/fits_utils).')
    end if

    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    aryStatInfo = getFitsVecStatSingleTab(tab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(tab)

  end function getFitsVecStatSingleSet

  !---------------------------------------------------------------------
  function getFitsVecStatSingleSetSimp(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatSingleSet(inSet, colName, tabName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatSingleSetSimp

  !---------------------------------------------------------------------
  function getFitsVecStatSingleTab(inTab, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(ColumnT) :: col
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    strFileInfoOut = "Column="//trim(colName)
    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    col = column(inTab, trim(colName), READ)

    aryStatInfo = getFitsVecStatSingleCol(col, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(col)

  end function getFitsVecStatSingleTab

  !---------------------------------------------------------------------
  function getFitsVecStatSingleTabSimp(inTab, colName, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32) :: minAreaIndex, maxAreaIndex
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatSingleTab(inTab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatSingleTabSimp

  !---------------------------------------------------------------------
  function getFitsVecStatSingleCol(inCol, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    real(single), pointer :: aryData(:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = dataType(inCol)	! dal
    if (iType .ne. REAL32) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into single in fits_utils/getFitsVecStatSingle"
      if (present(strFileInfo)) then
        errStr = trim(errStr)//" for "//trim(strFileInfo)
      else
        errStr = trim(errStr)//"."
      end if
      call message(AppMsg, Noisy)
    end if

    aryData => real32Data(inCol)
    aryStatInfo = getAryStatInfo(aryData, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

  end function getFitsVecStatSingleCol

  !---------------------------------------------------------------------
  function getFitsVecStatSingleColSimp(inCol, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatSingleCol(inCol, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatSingleColSimp


  !---------------------------------------------------------------------
  function getFitsVecStatInt32Name(setTabName, colName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo for a given FITS table.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: setTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    character(1024) :: setName, tabName
    logical :: noColonFound
    type(DataSetT) :: set
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(setTabName, setName, tabName, noColonFound, useBlock=.false.)
    if (noColonFound .and. (.not. present(extNum))) then
      ! The argument setTabName MUST contain the table name unless extNum is given.
      call error('noTabNameInGetFitsVecStat', 'In ssclib/fits_utils, for setTabName='//trim(setTabName))
    end if

    strFileInfoOut = "FileName="//trim(setName)
    set = dataSet(trim(setName), READ)

    if (trim(tabName) == '') then
      aryStatInfo = getFitsVecStatInt32Set(set, colName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    else
      aryStatInfo = getFitsVecStatInt32Set(set, colName, tabName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    end if

    call release(set)

  end function getFitsVecStatInt32Name

  !---------------------------------------------------------------------
  function getFitsVecStatInt32NameSimp(fitsSetTabName, colName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: fitsSetTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt32Name(fitsSetTabName, colName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo)

  end function getFitsVecStatInt32NameSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt32Set(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.
  ! Note if tabName is an empty string, it is ignored.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(TableT) :: tab
    logical :: isTableDefined
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    isTableDefined = .false.
    if (present(tabName)) then
      if (trim(tabName) .ne. '') then
        isTableDefined = .true.
        tab = table(inSet, trim(tabName))
        strFileInfoOut = "Table="//trim(tabName)
      end if
    end if

    if (isTableDefined) then
      ! Do nothing
    else if (present(extNum)) then
      tab = table(inSet, extNum)
      write(strFileInfoOut,'("Table-Ext=",i03)') extNum
    else
      ! Neither tabName nor extNum is specified.
      call error('noTabNameExtNum', 'Contact the code developper (ssclib/fits_utils).')
    end if

    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    aryStatInfo = getFitsVecStatInt32Tab(tab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(tab)

  end function getFitsVecStatInt32Set

  !---------------------------------------------------------------------
  function getFitsVecStatInt32SetSimp(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt32Set(inSet, colName, tabName, extNum, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt32SetSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt32Tab(inTab, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(ColumnT) :: col
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    strFileInfoOut = "Column="//trim(colName)
    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    col = column(inTab, trim(colName), READ)

    aryStatInfo = getFitsVecStatInt32Col(col, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(col)

  end function getFitsVecStatInt32Tab

  !---------------------------------------------------------------------
  function getFitsVecStatInt32TabSimp(inTab, colName, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt32Tab(inTab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt32TabSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt32Col(inCol, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32), pointer :: aryData(:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = dataType(inCol)	! dal
    if (iType .ne. INTEGER32) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int32 in fits_utils/getFitsVecStatInt32"
      if (present(strFileInfo)) then
        errStr = trim(errStr)//" for "//trim(strFileInfo)
      else
        errStr = trim(errStr)//"."
      end if
      call message(AppMsg, Noisy)
    end if

    aryData => int32Data(inCol)
    aryStatInfo = getAryStatInfo(aryData, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

  end function getFitsVecStatInt32Col

  !---------------------------------------------------------------------
  function getFitsVecStatInt32ColSimp(inCol, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt32Col(inCol, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt32ColSimp


  !---------------------------------------------------------------------
  function getFitsVecStatInt16Name(setTabName, colName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo for a given FITS table.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: setTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    character(1024) :: setName, tabName
    logical :: noColonFound
    type(DataSetT) :: set
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(setTabName, setName, tabName, noColonFound, useBlock=.false.)
    if (noColonFound .and. (.not. present(extNum))) then
      ! The argument setTabName MUST contain the table name unless extNum is given.
      call error('noTabNameInGetFitsVecStat', 'In ssclib/fits_utils, for setTabName='//trim(setTabName))
    end if

    strFileInfoOut = "FileName="//trim(setName)
    set = dataSet(trim(setName), READ)

    if (trim(tabName) == '') then
      aryStatInfo = getFitsVecStatInt16Set(set, colName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    else
      aryStatInfo = getFitsVecStatInt16Set(set, colName, tabName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    end if

    call release(set)

  end function getFitsVecStatInt16Name

  !---------------------------------------------------------------------
  function getFitsVecStatInt16NameSimp(fitsSetTabName, colName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: fitsSetTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt16Name(fitsSetTabName, colName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo)

  end function getFitsVecStatInt16NameSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt16Set(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.
  ! Note if tabName is an empty string, it is ignored.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(TableT) :: tab
    logical :: isTableDefined
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    isTableDefined = .false.
    if (present(tabName)) then
      if (trim(tabName) .ne. '') then
        isTableDefined = .true.
        tab = table(inSet, trim(tabName))
        strFileInfoOut = "Table="//trim(tabName)
      end if
    end if

    if (isTableDefined) then
      ! Do nothing
    else if (present(extNum)) then
      tab = table(inSet, extNum)
      write(strFileInfoOut,'("Table-Ext=",i03)') extNum
    else
      ! Neither tabName nor extNum is specified.
      call error('noTabNameExtNum', 'Contact the code developper (ssclib/fits_utils).')
    end if

    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    aryStatInfo = getFitsVecStatInt16Tab(tab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(tab)

  end function getFitsVecStatInt16Set

  !---------------------------------------------------------------------
  function getFitsVecStatInt16SetSimp(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt16Set(inSet, colName, tabName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt16SetSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt16Tab(inTab, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(ColumnT) :: col
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    strFileInfoOut = "Column="//trim(colName)
    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    col = column(inTab, trim(colName), READ)

    aryStatInfo = getFitsVecStatInt16Col(col, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(col)

  end function getFitsVecStatInt16Tab

  !---------------------------------------------------------------------
  function getFitsVecStatInt16TabSimp(inTab, colName, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt16Tab(inTab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt16TabSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt16Col(inCol, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int16), pointer :: aryData(:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = dataType(inCol)	! dal
    if (iType .ne. INTEGER16) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int16 in fits_utils/getFitsVecStatInt16"
      if (present(strFileInfo)) then
        errStr = trim(errStr)//" for "//trim(strFileInfo)
      else
        errStr = trim(errStr)//"."
      end if
      call message(AppMsg, Noisy)
    end if

    aryData => int16Data(inCol)
    aryStatInfo = getAryStatInfo(aryData, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

  end function getFitsVecStatInt16Col

  !---------------------------------------------------------------------
  function getFitsVecStatInt16ColSimp(inCol, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt16Col(inCol, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt16ColSimp


  !---------------------------------------------------------------------
  function getFitsVecStatInt8Name(setTabName, colName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo for a given FITS table.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: setTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    character(1024) :: setName, tabName
    logical :: noColonFound
    type(DataSetT) :: set
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    call splitSetTabName(setTabName, setName, tabName, noColonFound, useBlock=.false.)
    if (noColonFound .and. (.not. present(extNum))) then
      ! The argument setTabName MUST contain the table name unless extNum is given.
      call error('noTabNameInGetFitsVecStat', 'In ssclib/fits_utils, for setTabName='//trim(setTabName))
    end if

    strFileInfoOut = "FileName="//trim(setName)
    set = dataSet(trim(setName), READ)

    if (trim(tabName) == '') then
      aryStatInfo = getFitsVecStatInt8Set(set, colName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    else
      aryStatInfo = getFitsVecStatInt8Set(set, colName, tabName, extNum=extNum &
         , arMaskIn=arMaskIn &
         , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
         , valLower=valLower, valUpper=valUpper &
         , flagInfo=flagInfo, strFileInfo=strFileInfoOut)
    end if

    call release(set)

  end function getFitsVecStatInt8Name

  !---------------------------------------------------------------------
  function getFitsVecStatInt8NameSimp(fitsSetTabName, colName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: fitsSetTabName	! "FITS.ds" or "FITS.ds:TAB"
    character(*), intent(in) :: colName
    integer, intent(in), optional :: extNum
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt8Name(fitsSetTabName, colName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo)

  end function getFitsVecStatInt8NameSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt8Set(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.
  ! Note if tabName is an empty string, it is ignored.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(TableT) :: tab
    logical :: isTableDefined
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    isTableDefined = .false.
    if (present(tabName)) then
      if (trim(tabName) .ne. '') then
        isTableDefined = .true.
        tab = table(inSet, trim(tabName))
        strFileInfoOut = "Table="//trim(tabName)
      end if
    end if

    if (isTableDefined) then
      ! Do nothing
    else if (present(extNum)) then
      tab = table(inSet, extNum)
      write(strFileInfoOut,'("Table-Ext=",i03)') extNum
    else
      ! Neither tabName nor extNum is specified.
      call error('noTabNameExtNum', 'Contact the code developper (ssclib/fits_utils).')
    end if

    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    aryStatInfo = getFitsVecStatInt8Tab(tab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(tab)

  end function getFitsVecStatInt8Set

  !---------------------------------------------------------------------
  function getFitsVecStatInt8SetSimp(inSet, colName, tabName, extNum, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: inSet
    character(*), intent(in) :: colName
    character(*), intent(in), optional :: tabName	! or extNum
    integer, intent(in), optional :: extNum		! or tabName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt8Set(inSet, colName, tabName, extNum=extNum &
       , arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt8SetSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt8Tab(inTab, colName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    type(ColumnT) :: col
    character(1024) :: strFileInfoOut

    !-------------------------------------------------------------------
    ! Start

    strFileInfoOut = "Column="//trim(colName)
    if (present(strFileInfo)) then
      strFileInfoOut = trim(strFileInfo)//" "//trim(strFileInfoOut)
    end if

    col = column(inTab, trim(colName), READ)

    aryStatInfo = getFitsVecStatInt8Col(col, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfoOut)

    call release(col)

  end function getFitsVecStatInt8Tab

  !---------------------------------------------------------------------
  function getFitsVecStatInt8TabSimp(inTab, colName, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(TableT), intent(in) :: inTab
    character(*), intent(in) :: colName
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt8Tab(inTab, colName, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt8TabSimp

  !---------------------------------------------------------------------
  function getFitsVecStatInt8Col(inCol, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int8), pointer :: aryData(:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = dataType(inCol)	! dal
    if (iType .ne. INTEGER8) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int8 in fits_utils/getFitsVecStatInt8"
      if (present(strFileInfo)) then
        errStr = trim(errStr)//" for "//trim(strFileInfo)
      else
        errStr = trim(errStr)//"."
      end if
      call message(AppMsg, Noisy)
    end if

    aryData => int8Data(inCol)
    aryStatInfo = getAryStatInfo(aryData, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

  end function getFitsVecStatInt8Col

  !---------------------------------------------------------------------
  function getFitsVecStatInt8ColSimp(inCol, arMaskIn &
     , minAreaIndex, maxAreaIndex, valLower, valUpper &
     , flagInfo, strFileInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 1	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(ColumnT) :: inCol
    logical, intent(in), optional :: arMaskIn(:)
    integer(int32), intent(in) :: minAreaIndex, maxAreaIndex
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo
    character(*), intent(in), optional :: strFileInfo

    integer(int32) :: minAreaIndices(rankArin), maxAreaIndices(rankArin)

    !-------------------------------------------------------------------
    ! Start

    minAreaIndices = (/ minAreaIndex /) 
    maxAreaIndices = (/ maxAreaIndex /) 

    aryStatInfo = getFitsVecStatInt8Col(inCol, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper &
       , flagInfo=flagInfo, strFileInfo=strFileInfo)

  end function getFitsVecStatInt8ColSimp


  !---------------------------------------------------------------------
  function getFitsImgStatDoubleName(imageSetName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: imageSetName
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    real(double), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(trim(imageSetName))
    if (iType .ne. REAL64) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into double in fits_utils/getFitsImgStatDouble."
      call message(AppMsg, Noisy)
    end if

    call readArrayData(imageSetName, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatDoubleName


  !---------------------------------------------------------------------
  function getFitsImgStatDoubleSet(imageSet, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoDoubleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: imageSet
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    real(double), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(imageSet)
    if (iType .ne. REAL64) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into double in fits_utils/getFitsImgStatDouble."
      call message(AppMsg, Noisy)
    end if

    call readArrayData(imageSet, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatDoubleSet


  !---------------------------------------------------------------------
  function getFitsImgStatSingleName(imageSetName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: imageSetName
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    real(single), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(trim(imageSetName))
    if (iType .ne. REAL32) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into single in fits_utils/getFitsImgStatSingle."
      call message(AppMsg, Verbose)
    end if

    call readArrayData(imageSetName, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatSingleName


  !---------------------------------------------------------------------
  function getFitsImgStatSingleSet(imageSet, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoSingleT) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: imageSet
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(single), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    real(single), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(imageSet)
    if (iType .ne. REAL32) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into single in fits_utils/getFitsImgStatSingle."
      call message(AppMsg, Verbose)
    end if

    call readArrayData(imageSet, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatSingleSet


  !---------------------------------------------------------------------
  function getFitsImgStatInt32Name(imageSetName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: imageSetName
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType
    integer :: tmpVerboseLevel

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(trim(imageSetName))
    if (iType .ne. INTEGER32) then
      if (iType == REAL32 .or. iType == REAL64) then
        tmpVerboseLevel = Verbose
      else
        tmpVerboseLevel = Noisy
      end if
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int32 in fits_utils/getFitsImgStatInt32."
      call message(AppMsg, tmpVerboseLevel)
    end if

    call readArrayData(imageSetName, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt32Name


  !---------------------------------------------------------------------
  function getFitsImgStatInt32Set(imageSet, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt32T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: imageSet
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int32), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType
    integer :: tmpVerboseLevel

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(imageSet)
    if (iType .ne. INTEGER32) then
      if (iType == REAL32 .or. iType == REAL64) then
        tmpVerboseLevel = Verbose
      else
        tmpVerboseLevel = Noisy
      end if
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int32 in fits_utils/getFitsImgStatInt32."
      call message(AppMsg, tmpVerboseLevel)
    end if

    call readArrayData(imageSet, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt32Set


  !---------------------------------------------------------------------
  function getFitsImgStatInt16Name(imageSetName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: imageSetName
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int16), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType
    integer :: tmpVerboseLevel

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(trim(imageSetName))
    if (iType .ne. INTEGER16) then
      if (iType == REAL32 .or. iType == REAL64 .or. iType == INTEGER32) then
        tmpVerboseLevel = Verbose
      else
        tmpVerboseLevel = Noisy
      end if
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int16 in fits_utils/getFitsImgStatInt16."
      call message(AppMsg, tmpVerboseLevel)
    end if

    call readArrayData(imageSetName, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt16Name


  !---------------------------------------------------------------------
  function getFitsImgStatInt16Set(imageSet, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt16T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: imageSet
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int16), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType
    integer :: tmpVerboseLevel

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(imageSet)
    if (iType .ne. INTEGER16) then
      if (iType == REAL32 .or. iType == REAL64 .or. iType == INTEGER32) then
        tmpVerboseLevel = Verbose
      else
        tmpVerboseLevel = Noisy
      end if
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int16 in fits_utils/getFitsImgStatInt16."
      call message(AppMsg, tmpVerboseLevel)
    end if

    call readArrayData(imageSet, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt16Set


  !---------------------------------------------------------------------
  function getFitsImgStatInt8Name(imageSetName, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    character(*), intent(in) :: imageSetName
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int8), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(trim(imageSetName))
    if (iType .ne. INTEGER8) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int8 in fits_utils/getFitsImgStatInt8."
      call message(AppMsg, Verbose)
    end if

    call readArrayData(imageSetName, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt8Name


  !---------------------------------------------------------------------
  function getFitsImgStatInt8Set(imageSet, arMaskIn &
     , minAreaIndices, maxAreaIndices, valLower, valUpper &
     , flagInfo) result(aryStatInfo)
  !---------------------------------------------------------------------
  ! Return aryStatInfo.

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    type(aryStatInfoInt8T) :: aryStatInfo	! defined in ssclib/array_utils
    type(DataSetT), intent(in) :: imageSet
    logical, intent(in), optional :: arMaskIn(:,:)
    integer(int32), intent(in), optional :: minAreaIndices(rankArin), maxAreaIndices(rankArin)
    real(double), intent(in), optional :: valLower, valUpper
    type(aryStatInfoFlagT), intent(in), optional :: flagInfo

    integer(int8), pointer    :: aryImage(:,:) => null()
    integer :: iType
    character(strTypeLength) :: strType

    !-------------------------------------------------------------------
    ! Start

    iType = getDataType(imageSet)
    if (iType .ne. INTEGER8) then
      call getTypeName(iType, strType)		! dal_aux
      write(errStr, '(a)') "The data ("//trim(strType) &
           //") are converted into int8 in fits_utils/getFitsImgStatInt8."
      call message(AppMsg, Verbose)
    end if

    call readArrayData(imageSet, aryImage)	! dal_aux

    aryStatInfo = getAryStatInfo(aryImage, arMaskIn=arMaskIn &
       , minAreaIndices=minAreaIndices, maxAreaIndices=maxAreaIndices &
       , valLower=valLower, valUpper=valUpper, flagInfo=flagInfo)	! ssclib/array_utils

    deallocate(aryImage)

  end function getFitsImgStatInt8Set


  !---------------------------------------------------------------------
  function getAnnularMaskDoubleName(imageSetName, centX, centY &
     , rOuter, rInner, extNum) result(arMask)
  !---------------------------------------------------------------------
  ! Return arMask(:,:) (allocatable).

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    logical, allocatable :: arMask(:,:)	! result
    character(*), intent(in) :: imageSetName
    real(double), intent(in) :: centX, centY, rOuter
    real(double), intent(in), optional :: rInner	! 0 in default.
    integer(int32), intent(in), optional :: extNum	! Extention number of the image array in the input FITS file.

    type(DataSetT) :: imageSet

    !-------------------------------------------------------------------
    ! Start

    imageSet = dataSet(imageSetName, READ)
    arMask = getAnnularMaskDoubleSet(imageSet, centX, centY, rOuter &
       , rInner=rInner, extNum=extNum)
    call release(imageSet)

  end function getAnnularMaskDoubleName


  !---------------------------------------------------------------------
  function getAnnularMaskDoubleSet(imageSet, centX, centY &
     , rOuter, rInner, extNum) result(arMask)
  !---------------------------------------------------------------------
  ! Return arMask(:,:) (allocatable).

    implicit none
    integer, parameter :: rankArin = 2

    logical, allocatable :: arMask(:,:)	! result
    type(DataSetT), intent(in) :: imageSet
    real(double), intent(in) :: centX, centY, rOuter
    real(double), intent(in), optional :: rInner	! 0 in default.
    integer(int32), intent(in), optional :: extNum	! Extention number of the image array in the input FITS file.

    integer(int32) :: extNumLoc
    type(ArrayT) :: inArray
    integer(int32), pointer :: dimVector(:)

    !-------------------------------------------------------------------
    ! Start

    if (present(extNum)) then
       extNumLoc = extNum
    else
       extNumLoc = 0
    end if

    inArray = array(imageSet, extNumLoc, READ)
    dimVector => dimensions(inArray)	! Range: (0:1)
    if (size(dimVector) .ne. 2) then
      call error('nonImageInput')
    end if
    arMask = getAnnularMaskAry(dimVector(0), dimVector(1), centX, centY, rOuter &
       , rInner=rInner)	! defined in ssclib/array_utils
    call release(inArray)

  end function getAnnularMaskDoubleSet

  !---------------------------------------------------------------------
  function getAnnularMaskSingleName(imageSetName, centX, centY &
     , rOuter, rInner, extNum) result(arMask)
  !---------------------------------------------------------------------
  ! Return arMask(:,:) (allocatable).

    implicit none
    integer, parameter :: rankArin = 2	! = size(lbound(arin))

    logical, allocatable :: arMask(:,:)	! result
    character(*), intent(in) :: imageSetName
    real(single), intent(in) :: centX, centY, rOuter
    real(single), intent(in), optional :: rInner	! 0 in default.
    integer(int32), intent(in), optional :: extNum	! Extention number of the image array in the input FITS file.

    type(DataSetT) :: imageSet

    !-------------------------------------------------------------------
    ! Start

    imageSet = dataSet(imageSetName, READ)
    arMask = getAnnularMaskSingleSet(imageSet, centX, centY, rOuter &
       , rInner=rInner, extNum=extNum)
    call release(imageSet)

  end function getAnnularMaskSingleName


  !---------------------------------------------------------------------
  function getAnnularMaskSingleSet(imageSet, centX, centY &
     , rOuter, rInner, extNum) result(arMask)
  !---------------------------------------------------------------------
  ! Return arMask(:,:) (allocatable).

    implicit none
    integer, parameter :: rankArin = 2

    logical, allocatable :: arMask(:,:)	! result
    type(DataSetT), intent(in) :: imageSet
    real(single), intent(in) :: centX, centY, rOuter
    real(single), intent(in), optional :: rInner	! 0 in default.
    integer(int32), intent(in), optional :: extNum	! Extention number of the image array in the input FITS file.

    integer(int32) :: extNumLoc
    type(ArrayT) :: inArray
    integer(int32), pointer :: dimVector(:)

    !-------------------------------------------------------------------
    ! Start

    if (present(extNum)) then
       extNumLoc = extNum
    else
       extNumLoc = 0
    end if

    inArray = array(imageSet, extNumLoc, READ)
    dimVector => dimensions(inArray)	! Range: (0:1)
    if (size(dimVector) .ne. 2) then
      call error('nonImageInput')
    end if
    if (present(rInner)) then 
      arMask = getAnnularMaskAry(dimVector(0), dimVector(1) &
         , real(centX, double), real(centY, double) &
         , real(rOuter, double), rInner=real(rInner, double))	! defined in ssclib/array_utils
    else
      arMask = getAnnularMaskAry(dimVector(0), dimVector(1) &
         , real(centX, double), real(centY, double) &
         , real(rOuter, double))	! defined in ssclib/array_utils
    end if
    call release(inArray)

  end function getAnnularMaskSingleSet

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module fits_utils
