! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testfitsutils
  use dal
  use errorhandling
  use types
  use dal_aux	! in ssclib
  use fits_utils ! in this package

  implicit none

  integer, parameter :: ich = 256
  character(ich), parameter :: dalTestSetName = 'outfiles/daltest001a.ds'
  character(ich), parameter :: imgTestSetName = 'outfiles/pn_2000.ds'

  call test_getAnnularMask	! This uses files created in other test executables in this directory.

contains

  !---------------------------------------------------------------------
  subroutine test_getAnnularMask
  !---------------------------------------------------------------------
  ! Tests of getAnnularMaskDoubleName() in fits_utils
  ! Tests mostly teken from test_getAnnularMaskAry() in ssclib/test/testarrayutils.f90 

    implicit none

    real(double) :: centX, centY, rOuter, rInner
    real(double) :: xi, xf, yi, yf, xio, xfo, yio, yfo
    logical, allocatable :: arMask(:,:)
    integer(int32) :: sizeXDef, sizeYDef, i
    ! type(DataSetT) :: set
    character(16)  :: errHead

    !-------------------------------------------------------------------
    ! Start

    ! set = dataSet(trim(dalTestSetName), READ)

    !----------------------------------------------------------
    errHead = "AnnularMask0010a"
    print *,"=== "//errHead//": File: Radius=10, centred at (330,370)."

    sizeXDef = 648	! see outfiles/pn_2000.ds
    sizeYDef = 648
    centX = 330
    centY = 370
    rOuter = 10
    ! rInner
    arMask = getAnnularMask(imgTestSetName, centX, centY, rOuter)

    ! Debug...
    ! do i=1,sizeYDef
    do i=int(centY-rOuter-1), int(centY+rOuter+1)
      print *,arMask(int(centX-rOuter-1):int(centX+rOuter+1),i)
    end do

    xi = centX-rOuter+1
    xf = centX+rOuter-1
    yi = centY-rOuter+1
    yf = centY+rOuter-1
    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (.not. all(arMask(int(xi+(xf-xi)*0.25):int(xi+(xf-xi)*0.75), &
                              int(yi+(yf-yi)*0.25):int(yi+(yf-yi)*0.75)))) then
      ! print *, arMask
      call error("err"//errHead, 'Centre: false')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer: true')
    end if

    deallocate(arMask)

    !----------------------------------------------------------
    errHead = "AnnularMask0020a"
    print *,"=== "//errHead//": File: Radii=5-10, centred at (330,370)."

    sizeXDef = 648	! see outfiles/pn_2000.ds
    sizeYDef = 648
    centX = 330
    centY = 370
    rOuter = 10
    rInner = 5
    arMask = getAnnularMask(imgTestSetName, centX, centY, rOuter, rInner)

    ! Debug...
    ! do i=1,sizeYDef
    do i=int(centY-rOuter-1), int(centY+rOuter+1)
      print *,arMask(int(centX-rOuter-1):int(centX+rOuter+1),i)
    end do

    xi = centX-rInner+1
    xf = centX+rInner-1
    yi = centY-rInner+1
    yf = centY+rInner-1
    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (any(arMask(int(xi+(xf-xi)*0.25):int(xi+(xf-xi)*0.75), &
                        int(yi+(yf-yi)*0.25):int(yi+(yf-yi)*0.75)))) then
      ! print *, arMask
      call error("err"//errHead, 'Centre: true')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer: true')
    end if

    xio = centX-rOuter+1
    xfo = centX+rOuter-1
    yio = centY-rOuter+1
    yfo = centY+rOuter-1
    if      (.not. all(arMask(int(xio):int(centX-rInner-1), int(centY)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Smaller-X): false')
    else if (.not. all(arMask(int(centX+rInner+1):int(xfo), int(centY)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Larger-X): false')
    else if (.not. all(arMask(int(centX), int(yio):int(centY-rInner-1)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Smaller-Y): false')
    end if

    deallocate(arMask)

    !----------------------------------------------------------
    errHead = "AnnularMask0030a"
    print *,"=== "//errHead//": File: Radii=5-10, centred at (330,370), extNum=0."

    sizeXDef = 648	! see outfiles/pn_2000.ds
    sizeYDef = 648
    centX = 330
    centY = 370
    rOuter = 10
    rInner = 5
    arMask = getAnnularMask(imgTestSetName, centX, centY, rOuter, rInner, extNum=0)

    ! ! Debug...
    ! ! do i=1,sizeYDef
    ! do i=int(centY-rOuter-1), int(centY+rOuter+1)
    !   print *,arMask(int(centX-rOuter-1):int(centX+rOuter+1),i)
    ! end do

    xi = centX-rInner+1
    xf = centX+rInner-1
    yi = centY-rInner+1
    yf = centY+rInner-1
    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    end if

    deallocate(arMask)


    !!!  The following is commented out because it is expected to raise an error.
    !
    ! !----------------------------------------------------------
    ! errHead = "AnnularMask0040a"
    ! print *,"=== "//errHead//": File: Radii=5-10, centred at (330,370), extNum=1 => error(invalidBlockAccess)"
    ! 
    ! arMask = getAnnularMask(imgTestSetName, centX, centY, rOuter, rInner, extNum=1)	! => error(invalidBlockAccess)
    ! deallocate(arMask)


    !----------------------------------------------------------
    errHead = "AnnularMask0120a"
    print *,"=== "//errHead//": Single: File: Radii=5-10, centred at (330,370)."

    sizeXDef = 648	! see outfiles/pn_2000.ds
    sizeYDef = 648
    centX = 330
    centY = 370
    rOuter = 10
    rInner = 5
    arMask = getAnnularMask(imgTestSetName, real(centX,single), real(centY,single) &
       , real(rOuter,single), real(rInner,single))

    ! ! Debug...
    ! ! do i=1,sizeYDef
    ! do i=int(centY-rOuter-1), int(centY+rOuter+1)
    !   print *,arMask(int(centX-rOuter-1):int(centX+rOuter+1),i)
    ! end do

    xi = centX-rInner+1
    xf = centX+rInner-1
    yi = centY-rInner+1
    yf = centY+rInner-1
    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (any(arMask(int(xi+(xf-xi)*0.25):int(xi+(xf-xi)*0.75), &
                        int(yi+(yf-yi)*0.25):int(yi+(yf-yi)*0.75)))) then
      ! print *, arMask
      call error("err"//errHead, 'Centre: true')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer: true')
    end if

    xio = centX-rOuter+1
    xfo = centX+rOuter-1
    yio = centY-rOuter+1
    yfo = centY+rOuter-1
    if      (.not. all(arMask(int(xio):int(centX-rInner-1), int(centY)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Smaller-X): false')
    else if (.not. all(arMask(int(centX+rInner+1):int(xfo), int(centY)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Larger-X): false')
    else if (.not. all(arMask(int(centX), int(yio):int(centY-rInner-1)))) then
      ! print *, arMask
      call error("err"//errHead, 'Annular(Smaller-Y): false')
    end if

    deallocate(arMask)

  end subroutine test_getAnnularMask

end program testfitsutils
